@@ -2,8 +2,9 @@
  *            expenses-gtk.c
  *
  *  Mon Nov 21 22:56:18 2005
- *  Copyright  2005  Neil Williams
- *  linux@codehelp.co.uk
+ *  Copyright  2005  Neil Williams <linux@codehelp.co.uk>
+ *  Copyright (C) 2002, 2003, 2004 Philip Blundell <philb@gnu.org>
+ *  Copyright (C) 2005, 2006 Florian Boor <florian@kernelconcepts.de>
  ****************************************************************************/
 /*
     This package is free software; you can redistribute it and/or modify
@@ -33,6 +34,8 @@
 #include <gpe/pixmaps.h>
 #include <gpe/init.h>
 #include <gpe/pim-categories.h>
+#include <gpe/pim-categories-ui.h>
+#include <gpe/spacing.h>
 #include "expenses-gtk.h"
 #include "qof-main.h"
 #include "gpe-expenses.h"
@@ -73,6 +76,7 @@ compare_cache (gchar * text_entry, QofEntity * ent, const QofParam * param)
 {
 	gchar * check;
 	check = qof_util_param_to_string (ent, param);
+	g_return_if_fail (check);
 	if (safe_strcmp(check, text_entry))
 	{
 		qof_util_param_edit ((QofInstance*)ent, param);
@@ -82,8 +86,7 @@ compare_cache (gchar * text_entry, QofEntity * ent, const QofParam * param)
 	if (text_entry)
 		g_free (text_entry);
 	text_entry = NULL;
-	if (check)
-		g_free (check);
+	g_free (check);
 }
 
 /** \brief check each parameter and update backend
@@ -133,9 +136,8 @@ edit_ok_clicked(GtkWidget G_GNUC_UNUSED *widget, gpointer data)
 		(gtk_combo_box_get_active(context->payment_list))), ent, 
 		qof_class_get_parameter(GPE_QOF_EXPENSES, EXP_PAYMENT));
 	/* exp_category */
-	compare_cache (g_strdup_printf ("%d", 
-		gtk_combo_box_get_active (context->categories)), ent, 
-		qof_class_get_parameter (GPE_QOF_EXPENSES, EXP_CATEGORY));
+	compare_cache (g_strdup (gtk_label_get_text (GTK_LABEL(context->cat_label))),
+		ent, qof_class_get_parameter (GPE_QOF_EXPENSES, EXP_CATEGORY));
 	/* exp_currency */
 	mnemonic = gtk_combo_box_get_active_text (context->currency_list);
 	currency = gpe_currency_lookup_name ((QofInstance*)ent, mnemonic);
@@ -144,6 +146,7 @@ edit_ok_clicked(GtkWidget G_GNUC_UNUSED *widget, gpointer data)
 		compare_cache (g_strdup_printf ("%d", currency->pq_code), ent, 
 			qof_class_get_parameter (GPE_QOF_EXPENSES, EXP_CURRENCY));
 	gtk_widget_destroy(context->window);
+	context->entity = NULL;
 	exp_refresh_list(context);
 }
 
@@ -156,6 +159,7 @@ edit_cancel_clicked(GtkWidget G_GNUC_UNUSED *widget, gpointer data)
 	g_return_if_fail(context);
 	exp_refresh_list(context);
 	gtk_widget_destroy(context->window);
+	context->entity = NULL;
 }
 
 static void
@@ -169,6 +173,7 @@ edit_delete_clicked(GtkWidget G_GNUC_UNUSED *widget, gpointer data)
 	qof_entity_release(context->entity);
 	exp_refresh_list(context);
 	gtk_widget_destroy(context->window);
+	context->entity = NULL;
 }
 
 static void
@@ -183,16 +188,101 @@ create_currency_list(gpointer G_GNUC_UNUSED key, gpointer value, gpointer user_d
 		curr->mnemonic);
 }
 
-static void
-cat_populate (gpointer data, gpointer user_data)
+/* ported from gpe-contacts */
+static gchar *
+build_categories_string (GSList * catlist)
 {
-	struct gpe_pim_category * pim;
+	guint id;
+	GSList * iter;
+	const gchar *cat, *col;
+	gchar *s = NULL;
+
+	if (!catlist) { return g_strdup(""); }
+	for (iter = catlist; iter; iter = iter->next)
+	{
+		id = GPOINTER_TO_INT(iter->data);
+		cat = gpe_pim_category_name (id);
+		col = gpe_pim_category_colour (id);
+
+		if (cat)
+		{
+			if (s)
+			{
+				/* append as a comma-separated list */
+				gchar *ns = 
+					col ? g_strdup_printf ("%s, <span underline=\"single\" "\
+					"underline_color=\"%s\">%s</span>", s, col, cat) 
+					: g_strdup_printf ("%s, %s", s, cat);
+				g_free (s);
+				s = ns;
+			}
+			else
+				s = col ? g_strdup_printf ("<span underline=\"single\" "\
+					"underline_color=\"%s\">%s</span>", col, cat) 
+				: g_strdup (cat);
+		}
+	}
+
+	g_slist_free (catlist);
+
+	return s;
+}
+
+static void
+set_selected_category (GtkWidget G_GNUC_UNUSED *ui, GSList *selected, gpointer user_data)
+{
 	GpeExpenseData * context;
+	gchar * str;
 
-	pim = (struct gpe_pim_category*)data;
 	context = (GpeExpenseData*)user_data;
-	gtk_combo_box_insert_text (context->categories, pim->id, 
-		pim->name);
+	g_return_if_fail (context);
+
+	str = build_categories_string (selected);
+	if (str)
+	{
+		gtk_label_set_markup (GTK_LABEL (context->cat_label), str);
+		g_free (str);
+	}
+}
+
+/* based on gpe-contacts */
+static void
+on_categories_clicked (GtkButton G_GNUC_UNUSED *button, gpointer user_data)
+{
+	GtkWidget *w;
+
+	w = gpe_pim_categories_dialog (gpe_pim_categories_list (), 
+			G_CALLBACK (set_selected_category), user_data);
+}
+
+static gint
+cat_compare (gconstpointer gpe_cat, gconstpointer qof_name)
+{
+	struct gpe_pim_category * c;
+
+	c = (struct gpe_pim_category *) gpe_cat;
+	return safe_strcasecmp (gpe_pim_category_name (c->id), qof_name);
+}
+
+/* merge the QOF category list into the GPE PIM category list */
+static void
+cat_populate (const gchar * cat_name)
+{
+	GSList * cat_list, *match;
+	guint max_list;
+
+	match = NULL;
+	cat_list = gpe_pim_categories_list();
+	if (cat_list)
+	{
+		/* if the category name exists, leave it alone */
+		match = g_slist_find_custom (cat_list, cat_name, cat_compare);
+	}
+	if (!match)
+	{
+		DEBUG ("'%s' not found in GPE category list, adding.", cat_name);
+		gpe_pim_category_new (cat_name, &max_list);
+	}
 }
 
 static void 
@@ -201,7 +291,7 @@ edit_expense (GtkWidget G_GNUC_UNUSED *w, GpeExpenseData *context)
 	GtkWidget *table, *top_vbox;
 	GtkWidget *viewport, *scrolled_window;
 	GtkWidget *buttonbox, *buttonok, *buttoncancel, *buttondelete;
-	GtkWidget *cat_label, *type_label, *payment_label, *currency_label;
+	GtkWidget *type_label, *payment_label, *currency_label;
 	GtkWidget *city_label, *vendor_label, *note_label, *attendees_label;
 	GtkTextBuffer *buf;
 	guint gpe_spacing, pos, i;
@@ -223,7 +313,6 @@ edit_expense (GtkWidget G_GNUC_UNUSED *w, GpeExpenseData *context)
 	top_vbox = gtk_vbox_new (FALSE, 0);
 	context->text_view = gtk_text_view_new ();
 	buttonbox = gtk_hbox_new (FALSE, 0);
-	cat_label = gtk_label_new (_("Category:"));
 	type_label = gtk_label_new (_("Type:"));
 	payment_label = gtk_label_new (_("Payment:"));
 	currency_label = gtk_label_new (_("Currency:"));
@@ -231,10 +320,11 @@ edit_expense (GtkWidget G_GNUC_UNUSED *w, GpeExpenseData *context)
 	city_label = gtk_label_new(_("City:"));
 	note_label = gtk_label_new(_("Note:"));
 	attendees_label = gtk_label_new(_("Attendees"));
+	context->categories = gtk_button_new_with_label (_("Category"));
+	context->cat_label = gtk_label_new ("");
 	context->edit_type_list = GTK_COMBO_BOX(gtk_combo_box_new_text());
 	context->currency_list = GTK_COMBO_BOX(gtk_combo_box_new_text());
 	context->payment_list = GTK_COMBO_BOX(gtk_combo_box_new_text());
-	context->categories = GTK_COMBO_BOX(gtk_combo_box_new_text());
 	context->vendor_entry = GTK_ENTRY (gtk_entry_new());
 	context->city_entry = GTK_ENTRY (gtk_entry_new());
 	context->attendees_entry = GTK_ENTRY (gtk_entry_new());
@@ -248,14 +338,14 @@ edit_expense (GtkWidget G_GNUC_UNUSED *w, GpeExpenseData *context)
 	gtk_box_set_spacing (GTK_BOX(top_vbox), gpe_spacing);
 
 	gtk_window_set_default_size (GTK_WINDOW (context->window), 240, 320);
-        gtk_window_set_title (GTK_WINDOW (context->window), _("Expenses"));
+	gtk_window_set_title (GTK_WINDOW (context->window), _("Expenses"));
 
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
 				  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
 	gtk_viewport_set_shadow_type(GTK_VIEWPORT(viewport), GTK_SHADOW_NONE);
 	gtk_container_add (GTK_CONTAINER (viewport), table);
 	gtk_container_add (GTK_CONTAINER (scrolled_window), viewport);
-  
+
 	gtk_container_set_border_width(GTK_CONTAINER(buttonbox), 0);
 
 	gtk_table_set_col_spacings (GTK_TABLE(table), gpe_spacing);
@@ -273,7 +363,7 @@ edit_expense (GtkWidget G_GNUC_UNUSED *w, GpeExpenseData *context)
 	gtk_box_pack_start (GTK_BOX (buttonbox), buttoncancel, TRUE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (buttonbox), buttonok, TRUE, FALSE, 0);
 
-	gtk_misc_set_alignment (GTK_MISC (cat_label), 0.0, 0.0);
+	gtk_misc_set_alignment (GTK_MISC (context->cat_label), 0.0, 0.5);
 	gtk_misc_set_alignment (GTK_MISC (type_label), 0.0, 0.5);
 	gtk_misc_set_alignment (GTK_MISC (payment_label), 0.0, 0.5);
 	gtk_misc_set_alignment (GTK_MISC (currency_label), 0.0, 0.5);
@@ -290,16 +380,11 @@ edit_expense (GtkWidget G_GNUC_UNUSED *w, GpeExpenseData *context)
 
 	/* Category */
 	pos++;
-	gtk_table_attach (GTK_TABLE(table), cat_label, 0, 1, pos, pos+1, 
-				   GTK_FILL, GTK_FILL, 0, 0);
-	{
-		GSList * cat_list;
-		cat_list = gpe_pim_categories_list ();
-		g_slist_foreach (cat_list, cat_populate, context);
-		gtk_combo_box_set_active(context->categories, 0);
-		gtk_table_attach(GTK_TABLE(table), GTK_WIDGET(context->categories),
-			1, 3, pos, pos+1, GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
-	}
+	gtk_table_attach (GTK_TABLE(table), context->categories, 0, 1, pos, pos+1, 
+					  GTK_FILL, GTK_FILL, 0, 0);
+	/* the label comes after the button as the label *is* the data. */
+	gtk_table_attach (GTK_TABLE(table), context->cat_label, 2, 3, pos, pos+1, 
+					  GTK_FILL, GTK_FILL, 0, 0);
 	pos++;
 	/* Type of expense */
 	i = 0;
@@ -383,6 +468,16 @@ edit_expense (GtkWidget G_GNUC_UNUSED *w, GpeExpenseData *context)
 
 	/* use entity values to preset the edit window  */
 	i = 0;
+	param = qof_class_get_parameter(context->entity->e_type, EXP_CATEGORY);
+	text = param->param_getfcn(context->entity, param);
+	if(text) {
+		/* check the QOF categories against the GPE categories
+		 and add the OQF ones if they do not exist in GPE */
+		cat_populate (text);
+		PINFO ("setting cat_label=%s", text);
+		gtk_label_set_markup (GTK_LABEL(context->cat_label), text);
+	}
+
 	param = qof_class_get_parameter(context->entity->e_type, EXP_PAYMENT);
 	i = ExpensePaymentfromString(param->param_getfcn(context->entity, param));
 	gtk_combo_box_set_active(context->payment_list, i);
@@ -420,6 +515,8 @@ edit_expense (GtkWidget G_GNUC_UNUSED *w, GpeExpenseData *context)
 		gtk_text_view_set_buffer(GTK_TEXT_VIEW (context->text_view), buf);
 	}
 
+	g_signal_connect (G_OBJECT (context->categories), "clicked",
+					  G_CALLBACK (on_categories_clicked), context);
 	gpe_set_window_icon (context->window, "icon");
 	gtk_widget_show_all(context->window);
 	LEAVE (" ");
@@ -430,7 +527,10 @@ open_about_dialog (void)
 {
 	GtkAboutDialog* ab;
 	/* If you modify gpe-expenses, add your details here. */
-	const gchar * authors[2] = {"Neil Williams <linux@codehelp.co.uk>", NULL };
+	const gchar * authors[4] = {"Neil Williams <linux@codehelp.co.uk>\n"
+		"Philip Blundell <philb@gnu.org>\n"
+		"Florian Boor <florian@kernelconcepts.de>\n"
+		, NULL };
 
 	ab = GTK_ABOUT_DIALOG( gtk_about_dialog_new() );
 	gtk_about_dialog_set_copyright(ab, 
@@ -492,7 +592,6 @@ exp_show_entities(QofEntity *ent, gpointer data)
 	param = qof_class_get_parameter(GPE_QOF_EXPENSES, EXP_TYPE);
 	dis_string = param->param_getfcn(ent, param);
 	type = dgettext(LIBRARY_GETTEXT_PACKAGE, dis_string);
-//	qof_util_param_to_string(ent, param);
 	/* If Mileage, use ExpenseDistance,
 	else use ExpenseCustomCurrency->symbol */
 	if(0 == safe_strcmp(dis_string, "Mileage"))
@@ -663,7 +762,7 @@ change_amount (GtkCellRendererText G_GNUC_UNUSED *cell,
 	else 
 	{
 		qof_util_param_edit ((QofInstance*)context->entity, param);
-		numeric_setter(context->entity, gnc_numeric_zero ());
+		numeric_setter(context->entity, qof_numeric_zero ());
 		qof_util_param_commit ((QofInstance*)context->entity, param);
 	}
 	exp_refresh_list(context);
@@ -879,7 +978,7 @@ set_toolbar (GpeExpenseData *context)
 	g_signal_connect(G_OBJECT(edit_exp), "clicked",
 		G_CALLBACK (edit_expense), context);
 	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), edit_exp, -1);
-	
+
 	item = gtk_separator_tool_item_new();
 	gtk_tool_item_set_expand(GTK_TOOL_ITEM(item), FALSE);
 	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), item, -1);
